#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/mman.h>
#include <asm/page.h>
#include <arpa/inet.h>

#include "axi_ctrl.h"

#define PCI_DEVICE_LIST "/proc/bus/pci/devices"
#define BUFSIZE (1024)

#define VENDOR_ID (0x10ee)
#define DEVICE_ID (0x0007)

static int is_match_device(int id){
	if((id >> 16) == VENDOR_ID && (id & 0x00FF) == DEVICE_ID){
		return 1;
	}else{
		return 0;
	}
}

static int find_device(unsigned int *bar0)
{
	FILE *fp;
	char buf[BUFSIZE];
	const char * list = PCI_DEVICE_LIST;
	unsigned int bus, id, klass, b0;

	if((fp = fopen(list, "r")) == NULL){
	fprintf(stderr, "cannot open %s\n", list);
		return -1;
	}

	while(fgets(buf, BUFSIZE, fp) != NULL){
		sscanf(buf, "%x %x %x %x", &bus, &id, &klass, &b0);
		if(is_match_device(id) == 1){
			*bar0 = (b0 & 0xFFFFFF00);
			return 0;
		}
	}
	fclose(fp);
	return -1;
}

int open_ctrl_map(struct axi_ctrl_env *env)
{
	env->fd = open("/dev/mem", O_RDWR);
	unsigned int base;
	if(find_device(&base) < 0){
		fprintf(stderr,"cannot find device\n");
		return 1;
	}
	
	if(env->fd < 0){
		fprintf(stderr,"cannot open %s\n", "/dev/mem");
		return 1;
	}

	env->mmaped = mmap(0, AXI_CTRL_SIZE, PROT_READ|PROT_WRITE, MAP_SHARED, env->fd, base);
	if(env->mmaped == MAP_FAILED){
		fprintf(stderr,"cannot mmap\n");
		return 1;
	}
	env->pcie = (struct axi_pcie_ctrl*)(env->mmaped + AXI_PCIE_CTRL_OFFSET);
	env->cdma = (struct axi_cdma_ctrl*)(env->mmaped + AXI_CDMA_CTRL_OFFSET);
	env->gpio0 = (struct axi_gpio_ctrl*)(env->mmaped + AXI_GPIO0_CTRL_OFFSET);
	env->gpio1 = (struct axi_gpio_ctrl*)(env->mmaped + AXI_GPIO1_CTRL_OFFSET);
	return 0;
}

int close_ctrl_map(struct axi_ctrl_env *env)
{
	munmap(env->mmaped, AXI_CTRL_SIZE);
	close(env->fd);
	return 0;
}

#ifdef AXI_CTRL_MAIN
int main(int argc,char **argv)
{
	struct axi_ctrl_env env;
	open_ctrl_map(&env, FPGA_BAR0);
	close_ctrl_map(&env);
	return 0;
}
#endif
