#ifndef __AXI_CTRL_H__
#define __AXI_CTRL_H__

#define AXI_CTRL_SIZE (128*1024)	// 128K

#define AXI_PCIE_CTRL_OFFSET (0x00000000)
#define AXI_CDMA_CTRL_OFFSET (0x00001000)
#define AXI_GPIO0_CTRL_OFFSET (0x00002000)
#define AXI_GPIO1_CTRL_OFFSET (0x00003000)

struct axi_pcie_ctrl{
	unsigned char pcie_configuration_header[0x128]; 
	unsigned int vsec_cap;			// 0x128: Vendor-Specific (VSEC) Enhanced Capability
	unsigned int vsec_header;		// 0x12C: VSEC Header
	unsigned int bridge_info;		// 0x130: Bridge Info
	unsigned int bridge_status;		// 0x134: Bridge Status and Control 
	unsigned int interrupt_decode;		// 0x138: Interrupt Decode
	unsigned int interrupt_mask;		// 0x13C: Interrupt Mask
	unsigned int bus_location;		// 0x140: Bus Location
	unsigned int phy_status;		// 0x144: Physical-Side Interface(PHY) Status/Control
	unsigned int rp_status;			// 0x148: Root Port Status/Control
	unsigned int rp_msi_base1;		// 0x14C: Root Port MSI Base 1
	unsigned int rp_msi_base2;		// 0x150: Root Port MSI Base 2
	unsigned int rp_error;			// 0x154: Root Port Error FIFO Read
	unsigned int rp_interrupt_fifo_1;	// 0x158: Root Port Interrupt FIFO Read 1
	unsigned int rp_interrupt_fifo_2;	// 0x15C: Root Port Interrupt FIFO Read 2
	unsigned char reserved[160];		// 0x160-0x1FF: Reserved (zeros returned on read)
	unsigned int vsec_cap_2;		// 0x200: VSEC Capability 2
	unsigned int vsec_header_2;		// 0x204: VSEC Header 2
	unsigned int axibar2pciebar_0U;		// 0x208-0x234: AXI Base Address Translation Config. Registers
	unsigned int axibar2pciebar_0L;
	unsigned int axibar2pciebar_1U;
	unsigned int axibar2pciebar_1L;
	unsigned int axibar2pciebar_2U;
	unsigned int axibar2pciebar_2L;
	unsigned int axibar2pciebar_3U;
	unsigned int axibar2pciebar_3L;
	unsigned int axibar2pciebar_4U;
	unsigned int axibar2pciebar_4L;
	unsigned int axibar2pciebar_5U;
	unsigned int axibar2pciebar_5L;
};

struct axi_cdma_ctrl{
	unsigned int cdmacr;			// 00h: CDMA Control
	unsigned int cdmasr;			// 04h: CDMA Status
	unsigned int curdesc_pntr;		// 08h: Current Descriptor Pointer
	unsigned int reserved0;			// 0Ch: N/A
	unsigned int taildesc_pntr;		// 10h: Tail Descriptor Pointer
	unsigned int reserved1;			// 14h: N/A
	unsigned int sa;			// 18h: Source Address
	unsigned int reserved2;			// 1Ch: N/A
	unsigned int da;			// 20h: Desitination Address
	unsigned int reserved3;			// 24h: N/A
	unsigned int btt;			// 28h: Bytes to Transfer
};

struct axi_gpio_ctrl{
	unsigned int value;
	unsigned int dir;
};

struct axi_ctrl_env{
	int fd;
	char *mmaped;
	struct axi_pcie_ctrl *pcie;
	struct axi_cdma_ctrl *cdma;
	struct axi_gpio_ctrl *gpio0;
	struct axi_gpio_ctrl *gpio1;
};

int open_ctrl_map(struct axi_ctrl_env *env);
int close_ctrl_map(struct axi_ctrl_env *env);

#endif
